const express = require('express');
const router = express.Router();
const connection = require('./db');
const languageMessages = require('./language_messages');
const {DeviceTokenStore_1_Signal,cheakUseractiveDeactive} = require('./function_app_common');
const {fetchUserData } = require('./function_app');
// const {getNotificationArrSingle,oneSignalNotificationSendCall} = require("./notifications");
const jwt  = require('jsonwebtoken');
const dotenv = require("dotenv");
const crypto = require('crypto');
// const bcrypt = require('bcrypt');
const path = require('path');
const multer = require('multer');
const moment = require('moment');
const {mailer} = require("./mailer")
// const cookie = require('cookie-parser');

// image multer code start
dotenv.config();
const SECRET_KEY = process.env.USER_SECRET_KEY;

const storage = multer.diskStorage({

    destination: function (req, file, cb) {

      cb(null, './images/'); 

    },

    filename: function (req, file, cb) {

      const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);

      cb(null, file.fieldname + '-' + uniqueSuffix + path.extname(file.originalname));

    }

  });


  
const upload = multer({ storage: storage });


// video multer code 
const storage1 = multer.diskStorage({
    destination: function (req, file, cb) {
      cb(null, './videos/'); 
    },
    filename: function (req, file, cb) {
      const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
      cb(null, file.fieldname + '-' + uniqueSuffix + path.extname(file.originalname));
    }
  });
  const uploadVideo = multer({ storage: storage1 });

  // audio multer code 
const storage2 = multer.diskStorage({
    destination: function (req, file, cb) {
      cb(null, './audios/'); 
    },
    filename: function (req, file, cb) {
      const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
      cb(null, file.fieldname + '-' + uniqueSuffix + path.extname(file.originalname));
    }
  });
  const uploadAudio = multer({ storage: storage2 });


  
const signup = async (request, response) => {
    const { name, email, password, device_type, player_id, user_type, login_type, social_id } = request.body;
    try {
        if (!name) {
            return response.status(500).json({ success: false, msg: languageMessages.msg_empty_param,key:'name' });
        }
        if (!email) {
            return response.status(500).json({ success: false, msg: languageMessages.msg_empty_param,key:'email' });
        }
        if ( !device_type) {
            return response.status(500).json({ success: false, msg: languageMessages.msg_empty_param,key:'device_type' });
        }
        if (!player_id) {
            return response.status(500).json({ success: false, msg: languageMessages.msg_empty_param,key:'player_id' });
        }
        if (!user_type) {
            return response.status(500).json({ success: false, msg: languageMessages.msg_empty_param,key:'user_type' });
        }
        if (!login_type) {
            return response.status(500).json({ success: false, msg: languageMessages.msg_empty_param,key:'login_type' });
        }
        if (!password) {
            return response.status(500).json({ success: false, msg: languageMessages.msg_empty_param,key:'password' });
        }
      



       
        // var otp = await generateRandomOTP(4)
        // get random otp function end
        var otp = 1234;

        
        var google_id = "";
        var apple_id = "";
        var logintype1 = 0;
        if (login_type === 'google') {
            google_id = social_id;
            logintype1 = 1;
        } else if (login_type === 'apple') {
            apple_id = social_id;
            logintype1 = 2;
        }

        var sql = "SELECT user_id, mobile, otp_verify FROM user_master WHERE email = ? AND delete_flag = 0";
        connection.query(sql, [email], async (err, info) => {
            if (err) {
                console.error("Database error:", err);
                return response.status(500).json({ success: false, msg: languageMessages.internalServerError, key: "5" });
            } else {
                if (info.length > 0) {
                    var user_id = info[0].user_id;
                    if (info[0].otp_verify === 0) {
                        var sql1 = "UPDATE user_master SET otp = ?, updatetime = now(), login_type = ?,login_type_first = ? WHERE user_id = ?";
                        connection.query(sql1, [otp, logintype1,logintype1, user_id], (err, infomation) => {
                            if (err) {
                                console.error("Database error:", err);
                                return response.status(500).json({ success: false, msg: languageMessages.internalServerError, key: "1",error : err });
                            } else {
                                fetchUserData(user_id,async(error, userDataArray) => {
                                    if (error) {
                                        console.error('Error fetching user data:', error);
                                        return response.status(500).json({ success: false, msg: languageMessages.internalServerError });
                                    }

                                    // otp send start
                                var useremail = email;
                                var fromName = name;
                                var app_name= "Cinema App";
                                var message =  "Your Cinema otp is : ";
                                var subject =  "OTP Verification Code";
                                var title = "Signup";
                                const app_logo = "https://youngdecade.org/2024/cinema/webservice/logo/logo.png";
                                await mailer(useremail, fromName,app_name,message,subject,title,app_logo,otp).then(async data => {
                                    if(data.status === 'yes'){
                                        return response.status(200).json({ success: true, msg: languageMessages.otpsend, userDataArray });
                                    }
                                });
                                // otp send end
                                   
                                    // return response.status(200).json({ success: true, msg: languageMessages.otpsend, userDataArray });
                                });
                            }
                        });
                    } else {
                        return response.status(500).json({ success: false, msg: languageMessages.emailAlreadyExists, key: "6" });
                    }
                } else {
                    const hashedPassword = await hashPassword(password);
                    // var signup_step = 1;
                    var otp_verify = 0;
                    if(login_type != 'app'){
                        var otp_verify = 1
                    }
                    const updatetime = moment().format('YYYY-MM-DD HH:mm:ss');
                    const insertUser = "INSERT INTO user_master(otp_verify,name, email, password, user_type, otp,login_type, login_type_first, createtime, updatetime, google_id, apple_id,signup_step) VALUES (?,?, ?,?, ?,?, ?, ?,?,?, ?,?,?)";
                    var values = [otp_verify,name, email, hashedPassword, user_type, otp,logintype1, logintype1,updatetime,updatetime, google_id, apple_id, (login_type != 'app') ?  2 : 1];
                    connection.query(insertUser, values, (err, data) => {
                        if (err) {
                            console.error("Database error:", err);
                            return response.status(500).json({ success: false, msg: languageMessages.internalServerError, key: "1",error : err  });
                        } else {
                            const user_id = data.insertId;
                            DeviceTokenStore_1_Signal(user_id, device_type, player_id, function(result) {
                                console.error("Error storing device token:", result);
                                // return response.status(500).json({ success: false, msg: languageMessages.internalServerError, data: {} });
                            });
                            fetchUserData(user_id,async(error, userDataArray) => {
                                if (error) {
                                    console.error('Error fetching user data:', error);
                                    return response.status(500).json({ success: false, msg: languageMessages.internalServerError });
                                }

                               

                                var expireTime = {expiresIn : '1d'};
                                var token = jwt.sign({_id:email},SECRET_KEY,expireTime); 
                                
                                if(!token)
                                    response.render("error",{message:"Error while generating token inside recruiter login"});
                          
                                response.cookie('jwt_token',token,{maxAge:246060*1000,httpOnly:true});
                                // response.clearCookie('jwt_token');
// otp send start
                                var useremail = email;
                                var fromName = name;
                                var app_name= "Cinema App";
                                var message =  "Your Cinema otp is : ";
                                var subject =  "OTP Verification Code";
                                var title = "Signup";
                                const app_logo = "https://youngdecade.org/2024/cinema/webservice/logo/logo.png";
                                await mailer(useremail, fromName,app_name,message,subject,title,app_logo,otp).then(async data => {
                                    if(data.status === 'yes'){
                                        return response.status(200).json({ success: true, msg: languageMessages.signupSuccessfully, userDataArray:userDataArray,token:token});
                                    }
                                });
                                // otp send end
                                // return response.status(200).json({ success: true, msg: languageMessages.signupSuccessfully, userDataArray:userDataArray,token:token});
                            });
                        }
                    });
                }
            }
        });
    } catch (error) {
        console.error("Error:", error);
        return response.status(500).json({ success: false, msg: languageMessages.internalServerError, key: "3" });
    }
};

async function hashPassword(pass) {
    return crypto.createHash('sha256').update(pass).digest('hex');
}


// resend otp start

const resendOtp = (request, response) => {
    const { user_id } = request.body;
    console.log("user_id : ", user_id);
    try {
        if (!user_id) {
            return response.status(500).json({ success: false, msg: languageMessages.msg_empty_param,key:'user_id' });
        }
        // user check exists or not 

        const query1 = "SELECT user_id,active_flag,email,name FROM user_master WHERE delete_flag=0 and user_id = ?";
        const values1 = [user_id];
        connection.query(query1,values1,(queryError,queryResult)=>{
            if (queryError) {
                return response.status(500).json({ success: false, msg: languageMessages.internalServerError, key: "1" });
            }
            if(queryResult.length <= 0){
                return response.status(500).json({ success: false, msg: languageMessages.msgUserNotFound});
            }
            if(queryResult[0].active_flag === 0){
                return response.status(200).json({ success: false, msg: languageMessages.accountdeactivated });
            }
       
            let email = queryResult[0].email;
            let name = queryResult[0].name;

        // user check exists or not 
       // var otp = await generateRandomOTP(4)
        var otp = 1234;
        var sql = 'UPDATE user_master SET otp = ?, updatetime = NOW() WHERE user_id = ? AND delete_flag = 0';
        connection.query(sql, [otp, user_id], (error, result) => {
            if (error) {
                console.log(error, "error");
                return response.status(500).json({ success: false, msg: languageMessages.internalServerError, key: "2" });
            } else {
                fetchUserData(user_id,async(error, userDataArray) => {
                    if (error) {
                        console.error('Error fetching user data:', error);
                        return response.status(500).json({ success: false, msg: languageMessages.internalServerError });
                    }
                  
                    // otp send start
                    var useremail = email;
                    var fromName = name;
                    var app_name= "Cinema App";
                    var message =  "Your Cinema otp is : ";
                    var subject =  "OTP Verification Code";
                    var title = "Signup";
                    const app_logo = "https://youngdecade.org/2024/cinema/webservice/logo/logo.png";
                    await mailer(useremail, fromName,app_name,message,subject,title,app_logo,otp).then(async data => {
                        if(data.status === 'yes'){
                            return response.status(200).json({ success: true, msg: languageMessages.otpResendSuccess,userDataArray });
                        }
                    });

                    // return response.status(200).json({ success: true, msg: languageMessages.otpResendSuccess,userDataArray });
                })
               
            }
        });
    });
    } catch (error) {
        console.error("Error:", error);
        return response.status(500).json({ success: false, msg: languageMessages.internalServerError, key: "3" });
    }
};

// resend otp end


// otp verify start


const otpVerify = async (request, response) => {
    const { user_id, otp } = request.body;
    try {
        if (!user_id) {
            return response.status(500).json({ success: false, msg: languageMessages.msg_empty_param,key:'user_id' });
        }
        if (!otp) {
            return response.status(500).json({ success: false, msg: languageMessages.msg_empty_param,key:'otp' });
        }

        // user check exists or not 

        const query1 = "SELECT user_id,active_flag FROM user_master WHERE delete_flag=0 and user_id = ?";
        const values1 = [user_id];
        connection.query(query1, values1, async (queryError, queryResult) => {
            if (queryError) {
                return response.status(500).json({ success: false, msg: languageMessages.internalServerError, key: "1" });
            }
            if (queryResult.length <= 0) {
                return response.status(500).json({ success: false, msg: languageMessages.msgUserNotFound });
            }
            if (queryResult[0].active_flag === 0) {
                return response.status(200).json({ success: false, msg: languageMessages.accountdeactivated });
            }
            // user check exists or not 

            const checksql = "SELECT otp FROM user_master WHERE user_id = ? AND otp = ?";
            connection.query(checksql, [user_id, otp], async (err, data) => {
                if (err) {
                    return response.status(500).json({ success: false, msg: languageMessages.internalServerError });
                } else {
                    if (data.length > 0) {

                        const updatesql = 'UPDATE user_master SET otp_verify = 1, signup_step = 2, updatetime = now() WHERE user_id = ?';
                        connection.query(updatesql, [user_id], (err, information) => {
                            if (err) {
                                return response.status(500).json({ success: false, msg: languageMessages.internalServerError });
                            } else {
                                fetchUserData(user_id, async (error, userDataArray) => {
                                    if (error) {
                                        return response.status(500).json({ success: false, msg: languageMessages.internalServerError });
                                    }

                                    // send notification start 

                                    const user_id_notification = 1;
                                    const other_user_id_notification = user_id;
                                    const action = 'signup';
                                    const action_id = '0';
                                    const title = 'Signup';
                                    const title_2 = 'اشتراك';
                                    const title_3 = "S'inscrire";
                                    const title_4 = 'Зарегистрироваться';
                                    const title_5 = 'Inscribirse';
                                    const messages = 'Welcome to Ingle app';
                                    const message_2 = 'مرحبًا بك في تطبيق Ingle';
                                    const message_3 = "Bienvenue sur l'application Ingle";
                                    const message_4 = 'Добро пожаловать в приложение Ingle';
                                    const message_5 = 'Bienvenido a la aplicación Inglesa';
                                    const action_data = { user_id: user_id_notification, other_user_id: other_user_id_notification, action_id: action_id, action: action };

                                    getNotificationArrSingle(user_id_notification, other_user_id_notification, action, action_id, title, title_2, title_3, title_4, title_5, messages, message_2, message_3, message_4, message_5, action_data, async (notification_arr_check) => {
                                        let notification_arr_check_new = [notification_arr_check];

                                        if (notification_arr_check_new && notification_arr_check_new.length !== 0) {
                                            const notiSendStatus = await oneSignalNotificationSendCall(notification_arr_check_new);
                                        } else {
                                            console.log('Notification array is empty');
                                        }
                                    });

                                    // send notification end

                                    // add preference notification send start 

                                    const user_id_notification1 = 1;
                                    const other_user_id_notification1 = user_id;
                                    const action1 = 'add_preference';
                                    const action_id1 = '0';
                                    const title1 = 'Preference';
                                    const title_21 = 'التفضيل';
                                    const title_31 = 'Préférence';
                                    const title_41 = 'Предпочтение';
                                    const title_51 = 'Preferencia';
                                    const messages1 = 'Please add preference to get better match.';
                                    const message_21 = 'الرجاء إضافة التفضيل للحصول على تطابق أفضل.';
                                    const message_31 = 'Veuillez ajouter une préférence pour obtenir une meilleure correspondance.';
                                    const message_41 = 'Пожалуйста, добавьте предпочтение, чтобы получить лучшее соответствие.';
                                    const message_51 = 'Agregue preferencia para obtener una mejor coincidencia.';
                                    const action_data1 = { user_id: user_id_notification1, other_user_id: other_user_id_notification1, action_id: action_id1, action: action1 };

                                    getNotificationArrSingle(user_id_notification1, other_user_id_notification1, action1, action_id1, title1, title_21, title_31, title_41, title_51, messages1, message_21, message_31, message_41, message_51, action_data1, async (notification_arr_check1) => {
                                        let notification_arr_check_new1 = [notification_arr_check1];
                                        if (notification_arr_check_new1 && notification_arr_check_new1.length !== 0) {
                                            const notiSendStatus1 = await oneSignalNotificationSendCall(notification_arr_check_new1);
                                        } else {
                                            console.log('Notification array is empty');
                                        }
                                    });

                                    // add preference notification send end
                                    return response.status(200).json({ success: true, msg: languageMessages.otpverifysuccess, userDataArray });
                                })

                            }
                        });
                    } else {
                        return response.status(400).json({ success: false, msg: languageMessages.wrongotp });
                    }
                }
            });
        });
    } catch (error) {
        console.error("Error:", error);
        return response.status(500).json({ success: false, msg: languageMessages.internalServerError });
    }
}


// otp verify end

module.exports = {
    upload,
    signup,
    resendOtp
}